/* libguestfs generated file
 * WARNING: THIS FILE IS GENERATED FROM:
 *   generator/generator_*.ml
 * ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
 *
 * Copyright (C) 2009-2012 Red Hat Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <string.h>

#include "guestfs.h"
#include "guestfs-internal.h"
#include "guestfs-internal-actions.h"
#include "guestfs_protocol.h"

int
guestfs__internal_test_set_output (guestfs_h *g, const char *filename)
{
  FILE *fp;

  fp = fopen (filename, "w");
  if (fp == NULL) {
    perrorf (g, "cannot open output file %s\n", filename);
    return -1;
  }

  if (guestfs_internal_test_close_output (g) == -1) {
    fclose (fp);
    return -1;
  }

  g->test_fp = fp;

  return 0;
}

int
guestfs__internal_test_close_output (guestfs_h *g)
{
  if (g->test_fp != NULL) {
    if (fclose (g->test_fp) == EOF) {
      perrorf (g, "fclose");
      g->test_fp = NULL;
      return -1;
    }
    g->test_fp = NULL;
  }

  return 0;
}

static inline FILE *
get_fp (guestfs_h *g)
{
  if (g->test_fp)
    return g->test_fp;
  else
    return stdout;
}

static void
print_strings (guestfs_h *g, char *const *argv)
{
  FILE *fp = get_fp (g);
  size_t argc;

  fprintf (fp, "[");
  for (argc = 0; argv[argc] != NULL; ++argc) {
    if (argc > 0) fprintf (fp, ", ");
    fprintf (fp, "\"%s\"", argv[argc]);
  }
  fprintf (fp, "]\n");
}

/* The internal_test function prints its parameters to stdout or the
 * file set by internal_test_set_output.
 */
int
guestfs__internal_test (guestfs_h *g,
                        const char *str,
                        const char *optstr,
                        char *const *strlist,
                        int b,
                        int integer,
                        int64_t integer64,
                        const char *filein,
                        const char *fileout,
                        const char *bufferin,
                        size_t bufferin_size,
                        const struct guestfs_internal_test_argv *optargs)
{
  FILE *fp = get_fp (g);

  fprintf (fp, "%s\n", str);
  fprintf (fp, "%s\n", optstr ? optstr : "null");
  print_strings (g, strlist);
  fprintf (fp, "%s\n", b ? "true" : "false");
  fprintf (fp, "%d\n", integer);
  fprintf (fp, "%" PRIi64 "\n", integer64);
  fprintf (fp, "%s\n", filein);
  fprintf (fp, "%s\n", fileout);
  {
    size_t i;
    for (i = 0; i < bufferin_size; ++i)
      fprintf (fp, "<%02x>", bufferin[i]);
    fprintf (fp, "\n");
  }
  fprintf (fp, "obool: ");
  if (optargs->bitmask & GUESTFS_INTERNAL_TEST_OBOOL_BITMASK) {
    fprintf (fp, "%s\n", optargs->obool ? "true" : "false");
  } else {
    fprintf (fp, "unset\n");
  }
  fprintf (fp, "oint: ");
  if (optargs->bitmask & GUESTFS_INTERNAL_TEST_OINT_BITMASK) {
    fprintf (fp, "%i\n", optargs->oint);
  } else {
    fprintf (fp, "unset\n");
  }
  fprintf (fp, "oint64: ");
  if (optargs->bitmask & GUESTFS_INTERNAL_TEST_OINT64_BITMASK) {
    fprintf (fp, "%" PRIi64 "\n", optargs->oint64);
  } else {
    fprintf (fp, "unset\n");
  }
  fprintf (fp, "ostring: ");
  if (optargs->bitmask & GUESTFS_INTERNAL_TEST_OSTRING_BITMASK) {
    fprintf (fp, "%s\n", optargs->ostring);
  } else {
    fprintf (fp, "unset\n");
  }
  fprintf (fp, "ostringlist: ");
  if (optargs->bitmask & GUESTFS_INTERNAL_TEST_OSTRINGLIST_BITMASK) {
    print_strings (g, optargs->ostringlist);
  } else {
    fprintf (fp, "unset\n");
  }
  /* Java changes stdout line buffering so we need this: */
  fflush (fp);
  return 0;
}

/* Test normal return. */
int
guestfs__internal_test_rint (guestfs_h *g,
                             const char *val)
{
  int r;
  if (sscanf (val, "%d", &r) != 1) {
    error (g, "%s: expecting int argument", "internal_test_rint");
    return -1;
  }
  return r;
}

/* Test error return. */
int
guestfs__internal_test_rinterr (guestfs_h *g)
{
  error (g, "error");
  return -1;
}

/* Test normal return. */
int64_t
guestfs__internal_test_rint64 (guestfs_h *g,
                               const char *val)
{
  int64_t r;
  if (sscanf (val, "%" SCNi64, &r) != 1) {
    error (g, "%s: expecting int64 argument", "internal_test_rint64");
    return -1;
  }
  return r;
}

/* Test error return. */
int64_t
guestfs__internal_test_rint64err (guestfs_h *g)
{
  error (g, "error");
  return -1;
}

/* Test normal return. */
int
guestfs__internal_test_rbool (guestfs_h *g,
                              const char *val)
{
  return STREQ (val, "true");
}

/* Test error return. */
int
guestfs__internal_test_rboolerr (guestfs_h *g)
{
  error (g, "error");
  return -1;
}

/* Test normal return. */
const char *
guestfs__internal_test_rconststring (guestfs_h *g,
                                     const char *val)
{
  return "static string";
}

/* Test error return. */
const char *
guestfs__internal_test_rconststringerr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
const char *
guestfs__internal_test_rconstoptstring (guestfs_h *g,
                                        const char *val)
{
  return "static string";
}

/* Test error return. */
const char *
guestfs__internal_test_rconstoptstringerr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
char *
guestfs__internal_test_rstring (guestfs_h *g,
                                const char *val)
{
  return strdup (val);
}

/* Test error return. */
char *
guestfs__internal_test_rstringerr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
char **
guestfs__internal_test_rstringlist (guestfs_h *g,
                                    const char *val)
{
  char **strs;
  size_t n, i;
  if (sscanf (val, "%zu", &n) != 1) {
    error (g, "%s: expecting int argument", "internal_test_rstringlist");
    return NULL;
  }
  strs = safe_malloc (g, (n+1) * sizeof (char *));
  for (i = 0; i < n; ++i) {
    strs[i] = safe_malloc (g, 16);
    snprintf (strs[i], 16, "%zu", i);
  }
  strs[n] = NULL;
  return strs;
}

/* Test error return. */
char **
guestfs__internal_test_rstringlisterr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
struct guestfs_lvm_pv *
guestfs__internal_test_rstruct (guestfs_h *g,
                                const char *val)
{
  struct guestfs_lvm_pv *r;
  r = safe_calloc (g, sizeof *r, 1);
  return r;
}

/* Test error return. */
struct guestfs_lvm_pv *
guestfs__internal_test_rstructerr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
struct guestfs_lvm_pv_list *
guestfs__internal_test_rstructlist (guestfs_h *g,
                                    const char *val)
{
  struct guestfs_lvm_pv_list *r;
  uint32_t len;
  if (sscanf (val, "%" SCNu32, &len) != 1) {
    error (g, "%s: expecting uint32 argument", "internal_test_rstructlist");
    return NULL;
  }
  r = safe_calloc (g, sizeof *r, 1);
  r->len = len;
  r->val = safe_calloc (g, r->len, sizeof *r->val);
  for (size_t i = 0; i < r->len; i++) {
    r->val[i].pv_size = i;
  }
  return r;
}

/* Test error return. */
struct guestfs_lvm_pv_list *
guestfs__internal_test_rstructlisterr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
char **
guestfs__internal_test_rhashtable (guestfs_h *g,
                                   const char *val)
{
  char **strs;
  size_t n, i;
  if (sscanf (val, "%zu", &n) != 1) {
    error (g, "%s: expecting int argument", "internal_test_rhashtable");
    return NULL;
  }
  strs = safe_malloc (g, (n*2+1) * sizeof (*strs));
  for (i = 0; i < n; ++i) {
    strs[i*2] = safe_malloc (g, 16);
    strs[i*2+1] = safe_malloc (g, 16);
    snprintf (strs[i*2], 16, "%zu", i);
    snprintf (strs[i*2+1], 16, "%zu", i);
  }
  strs[n*2] = NULL;
  return strs;
}

/* Test error return. */
char **
guestfs__internal_test_rhashtableerr (guestfs_h *g)
{
  error (g, "error");
  return NULL;
}

/* Test normal return. */
char *
guestfs__internal_test_rbufferout (guestfs_h *g,
                                   const char *val,
                                   size_t *size_r)
{
  *size_r = strlen (val);
  return strdup (val);
}

/* Test error return. */
char *
guestfs__internal_test_rbufferouterr (guestfs_h *g,
                                      size_t *size_r)
{
  error (g, "error");
  return NULL;
}

